package com.company.demo;

import com.google.common.base.Strings;
import com.vaadin.flow.component.page.AppShellConfigurator;
import com.vaadin.flow.component.page.Push;
import com.vaadin.flow.server.PWA;
import com.vaadin.flow.theme.Theme;
import io.jmix.reports.ReportsProperties;
import io.jmix.reports.libintegration.JmixFieldFormatProvider;
import io.jmix.reports.libintegration.JmixFormatterFactory;
import io.jmix.reports.libintegration.JmixInlinersProvider;
import io.jmix.reports.libintegration.JmixOfficeIntegration;
import io.jmix.reports.yarg.util.groovy.Scripting;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.boot.context.event.ApplicationStartedEvent;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.boot.autoconfigure.jdbc.DataSourceProperties;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Primary;
import org.springframework.context.event.EventListener;
import org.springframework.core.env.Environment;
import io.jmix.reports.libintegration.CustomJmixFormatterFactory;

import javax.sql.DataSource;

@Push
@Theme(value = "demo")
@PWA(name = "Demo", shortName = "Demo")
@SpringBootApplication
public class DemoApplication implements AppShellConfigurator {

    @Autowired
    private Environment environment;
    @Autowired
    protected ReportsProperties reportsProperties;

    public static void main(String[] args) {
        SpringApplication.run(DemoApplication.class, args);
    }

    @Bean
    @Primary
    @ConfigurationProperties("main.datasource")
    DataSourceProperties dataSourceProperties() {
        return new DataSourceProperties();
    }

    @Bean
    @Primary
    @ConfigurationProperties("main.datasource.hikari")
    DataSource dataSource(final DataSourceProperties dataSourceProperties) {
        return dataSourceProperties.initializeDataSourceBuilder().build();
    }

    @EventListener
    public void printApplicationUrl(final ApplicationStartedEvent event) {
        LoggerFactory.getLogger(DemoApplication.class).info("Application started at "
                + "http://localhost:"
                + environment.getProperty("local.server.port")
                + Strings.nullToEmpty(environment.getProperty("server.servlet.context-path")));
    }

    @Primary
    @Bean("report_CustomFormatterFactory")
    public JmixFormatterFactory formatterFactory(JmixInlinersProvider inlinersProvider,
                                                 JmixFieldFormatProvider fieldFormatProvider,
                                                 Scripting scripting,
                                                 JmixOfficeIntegration officeIntegration) {
        CustomJmixFormatterFactory formatterFactory = new CustomJmixFormatterFactory();
        formatterFactory.setUseOfficeForDocumentConversion(reportsProperties.isUseOfficeForDocumentConversion());
        formatterFactory.setInlinersProvider(inlinersProvider);
        formatterFactory.setDefaultFormatProvider(fieldFormatProvider);
        formatterFactory.setOfficeIntegration(officeIntegration);
        formatterFactory.setScripting(scripting);
        return formatterFactory;
    }
}
