package io.jmix.reports.libintegration;

import io.jmix.reports.yarg.formatters.impl.CustomDocxFormatter;
import io.jmix.reports.yarg.formatters.impl.DocxFormatter;
import io.jmix.reports.yarg.formatters.impl.HtmlFormatter;
import io.jmix.reports.yarg.formatters.impl.XlsxFormatter;


public class CustomJmixFormatterFactory extends JmixFormatterFactory {

    public CustomJmixFormatterFactory() {
        super();
        FormatterCreator ftlCreator = factoryInput -> {
            HtmlFormatter htmlFormatter = beanFactory.getBean(JmixHtmlFormatter.class, factoryInput);
            htmlFormatter.setDefaultFormatProvider(defaultFormatProvider);
            htmlFormatter.setScripting(scripting);
            return htmlFormatter;
        };
        formattersMap.put("ftl", ftlCreator);
        formattersMap.put("html", ftlCreator);

        FormatterCreator docxCreator = factoryInput -> {
            DocxFormatter docxFormatter = beanFactory.getBean(CustomDocxFormatter.class, factoryInput);
            docxFormatter.setDefaultFormatProvider(defaultFormatProvider);
            if (useOfficeForDocumentConversion) {
                docxFormatter.setDocumentConverter(documentConverter);
            }
            docxFormatter.setHtmlImportProcessor(htmlImportProcessor);
            docxFormatter.setScripting(scripting);
            return docxFormatter;
        };

        formattersMap.put("docx", docxCreator);
        formattersMap.put("chart", factoryInput -> beanFactory.getBean(ChartFormatter.class, factoryInput));
        formattersMap.put("pivot", factoryInput -> beanFactory.getBean(PivotTableFormatter.class, factoryInput));

        FormatterCreator xlsxCreator = factoryInput -> {
            XlsxFormatter xlsxFormatter = beanFactory.getBean(JmixXlsxFormatter.class, factoryInput);
            xlsxFormatter.setDefaultFormatProvider(defaultFormatProvider);
            xlsxFormatter.setDocumentConverter(documentConverter);
            xlsxFormatter.setScripting(scripting);
            xlsxFormatter.setFormulasPostProcessingEvaluationEnabled(reportsProperties.isFormulasPostProcessingEvaluationEnabled());
            return xlsxFormatter;
        };
        formattersMap.put("xlsx", xlsxCreator);

        formattersMap.put("table", factoryInput -> beanFactory.getBean(JmixTableFormatter.class, factoryInput));
    }
}
